@@ -1,5 +1,4 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
+#!/usr/bin/env python3
 #
 # gpgsign.py - part of the FDroid server tools
 # Copyright (C) 2014, Ciaran Gultnieks, ciaran@ciarang.com
@@ -23,8 +22,8 @@ import glob
 from argparse import ArgumentParser
 import logging
 
-import common
-from common import FDroidPopen
+from . import common
+from .common import FDroidPopen
 
 config = None
 options = None
